import os
import json
import logging.config


def setup_logging(default_level=logging.INFO,
                  default_path="{}/logging.json".format(
                      os.getenv(
                          "LOG_DIR",
                          os.path.dirname(os.path.realpath(__file__)))),
                  env_key="LOG_CFG",
                  config_name=None):

    """
    Setup logging configuration
    """
    path = default_path
    file_name = default_path.split("/")[-1]
    if config_name:
        file_name = config_name

    path = "{}/{}".format(
                "/".join(default_path.split("/")[:-1]),
                file_name)

    value = os.getenv(env_key, None)
    if value:
        path = value
    if os.path.exists(path):
        with open(path, "rt") as f:
            config = json.load(f)
        logging.config.dictConfig(config)
        return

    else:
        cwd_path = os.getcwd() + "/network_pipeline/log/{}".format(
                                    file_name)
        if os.path.exists(cwd_path):
            with open(cwd_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        rels_path = os.getcwd() + "/../log/{}".format(
                                    file_name)
        if os.path.exists(rels_path):
            with open(rels_path, "rt") as f:
                config = json.load(f)
            logging.config.dictConfig(config)
            return
        else:
            logging.basicConfig(level=default_level)
            return
# end of setup_logging



#!/usr/bin/env python

import os
import sys
import logging
import json
import numpy as np
import pandas as pd
from celery_connectors.utils import ev
from network_pipeline.log.setup_logging import setup_logging
from keras.models import Sequential
from keras.layers import Dense
from sklearn.model_selection import train_test_split


setup_logging(config_name="modelers.json")
name = "keras-dnn"
log = logging.getLogger(name)
log.info("start - {}".format(name))


# seed for reproducing same results
seed = 9
np.random.seed(seed)

csv_file = ev(
    "CSV_FILE",
    "/tmp/merge_only_attack_scans.csv")
meta_file = ev(
    "CSV_META_FILE",
    "/tmp/merge_headers.json")

if not os.path.exists(csv_file):
    log.error(("missing csv_file={}")
              .format(csv_file))
    sys.exit(1)

predict_feature = ev(
    "PREDICT_FEATURE",
    "label_value")

log.info(("Loading csv={}")
         .format(csv_file))

# load csv file into pandas dataframe
df = pd.read_csv(csv_file)

# set up the test to train ratio
test_size = float(ev(
    "TEST_SIZE",
    "0.20"))
train_size = 1.0 - test_size

feature_to_process = []
ignore_features = [
    "label_name",
    "ip_src",   # need to make this an int
    "ip_dst",   # need to make this an int
    "eth_src",  # need to make this an int
    "eth_dst"   # need to make this an int
]
meta_data = {}

try:
    meta_data = json.loads(
        open(meta_file, "r").read()
    )
    if "post_proc_rules" in meta_data:
        if "drop_columns" in meta_data["post_proc_rules"]:
            for ign in meta_data["post_proc_rules"]["drop_columns"]:
                ignore_features.append(ign)
except Exception as e:
    log.error(("Failed building ignore_features: "
               "ignore_features={} meta={} meta_data={} "
               "ex={}")
              .format(ignore_features,
                      meta_file,
                      meta_data,
                      e))
# end of trying to lookup the meta data file
# for non-int/float features to ignore

for feature in df.columns.values:
    keep_it = True
    for ign in ignore_features:
        if feature == ign:
            keep_it = False
    if keep_it:
        feature_to_process.append(feature)
# end of for all features to process

log.info(("Predicting={} with features={} "
          "ignore_features={} records={}")
         .format(predict_feature,
                 feature_to_process,
                 ignore_features,
                 len(df.index)))

# split the data into training
(X_train, X_test, Y_train, Y_test) = \
        train_test_split(df[feature_to_process],
                         df[predict_feature],
                         test_size=test_size,
                         random_state=seed)

log.info(("splitting rows={} into "
          "X_train={} X_test={}"
          "Y_train={} Y_test={}")
         .format(len(df.index),
                 len(X_train),
                 len(X_test),
                 len(Y_train),
                 len(Y_test)))

log.info("creating sequential model")

# create the model
model = Sequential()
model.add(Dense(8,
                input_dim=len(feature_to_process),
                kernel_initializer="uniform",
                activation="relu"))
model.add(Dense(6,
                kernel_initializer="uniform",
                activation="relu"))
model.add(Dense(1,
                kernel_initializer="uniform",
                activation="sigmoid"))

log.info("compiling model")

# compile the model
model.compile(loss="binary_crossentropy",
              optimizer="adam",
              metrics=["accuracy"])

log.info("fitting model - please wait")

# fit the model
model.fit(X_train,
          Y_train,
          validation_data=(X_test, Y_test),
          epochs=200,
          batch_size=5,
          verbose=0)

# evaluate the model
scores = model.evaluate(X_test, Y_test)
log.info(("Accuracy: {}")
         .format(scores[1] * 100))



import logging
import socket
import time
from network_pipeline.log.setup_logging import setup_logging

setup_logging()
name = "connect-forwarder"
log = logging.getLogger(name)


def connect_forwarder(forward_host=None,
                      forward_port=None,
                      max_retries=-1,
                      sleep_interval=1.0):
    """connect_forwarder

    :param forward_host: host for receiving forwarded packets
    :param forward_port: port for the forwarded packets
    :param max_retries: retries, -1 = infinite
    :param sleep_interval: how often to retry in this loop
    """

    forward_skt = None
    retry_count = 0
    if max_retries == -1:
        retry_count = -2

    if forward_host and forward_port:
        while not forward_skt and \
              retry_count < max_retries:
            try:
                forward_skt = socket.socket()
                log.info(("connecting to forward={}:{}")
                         .format(forward_host,
                                 forward_port))
                forward_skt.connect((forward_host,
                                     forward_port))
                log.debug(("connected to forward={}:{}")
                          .format(forward_host,
                                  forward_port))
            except Exception as s:
                forward_skt = None
                log.error(("Failed to connect forward address={}:{} "
                           "with ex={}")
                          .format(forward_host,
                                  forward_port,
                                  s))
                if max_retries == -1:
                    retry_count = -2
                else:
                    retry_count += 1
            # end of try/ex
            time.sleep(sleep_interval)
        # end of setting up forward
    # end forward_host and forward_port

    return forward_skt
# end of connect_forwarder



import logging
from network_pipeline.consts import SOURCE
from network_pipeline.consts import FORWARD_EXCHANGE
from network_pipeline.consts import FORWARD_ROUTING_KEY
from network_pipeline.consts import FORWARD_QUEUE
from network_pipeline.log.setup_logging import setup_logging
from network_pipeline.utils import rnow
from network_pipeline.convert_pkt_to_json import convert_pkt_to_json
from network_pipeline.publisher import pub
import scapy.all as scapy


setup_logging()
name = "proc"
log = logging.getLogger(name)


def handle_packets(pk):
    """handle_packets

    :param pk: data packet that scapy sends in
    """

    log.info(("processing with pub={}")
             .format(pub))

    # get the lowest layer
    eth = pk.getlayer(scapy.Ether)

    should_forward = False
    send_msg = {"data": {},
                "created": rnow(),
                "source": SOURCE}

    if eth:
        # parse all layer frames under ethernet
        send_msg["data"] = convert_pkt_to_json(eth)
        should_forward = True
    else:
        log.error(("unsupported pk={}")
                  .format(pk))
    # end of if supported

    if should_forward:

        log.info("forwarding")

        # Publish the message:
        msg_sent = pub.publish(body=send_msg,
                               exchange=FORWARD_EXCHANGE,
                               routing_key=FORWARD_ROUTING_KEY,
                               queue=FORWARD_QUEUE,
                               serializer="json",
                               retry=True)

        log.info("done forwarding={}".format(msg_sent))

    # end of should_forward

# end of handle_packets



#!/usr/bin/env python

import logging
import pandas as pd
import glob
import copy
import random
from network_pipeline.consts import VALID
from network_pipeline.consts import INVALID
from network_pipeline.utils import ppj
from network_pipeline.utils import rnow
from network_pipeline.log.setup_logging import setup_logging
from celery_connectors.utils import ev


setup_logging(config_name="builders.json")
name = "builder"
log = logging.getLogger(name)
log.info("start - {}".format(name))


def find_all_headers(pipeline_files=[],
                     label_rules=None):
    log.info("find_all_headers - START")

    headers = ["src_file"]
    headers_dict = {"src_file": None}

    if label_rules:
        headers = ["src_file", "label_value", "label_name"]
        headers_dict = {"src_file": None,
                        "label_value": None,
                        "label_name": None}

    for c in pipeline_files:
        df = pd.read_csv(c)
        for h in df.columns.values:
            if h not in headers_dict:
                headers_dict[h] = "{}_{}".format(
                                    c,
                                    h)
                headers.append(h)
        # end for all headers in the file
    # end for all files to find common_headers

    log.info(("headers={}")
             .format(len(headers)))

    log.info("find_all_headers - END")
    return headers, headers_dict
# end of find_all_headers


def build_csv(
        pipeline_files=[],
        merge_file=None,
        output_file=None,
        post_proc_rules=None,
        label_rules=None,
        headers_filename="headers.json"):

    save_node = {
        "status": INVALID,
        "pipeline_files": pipeline_files,
        "post_proc_rules": post_proc_rules,
        "label_rules": label_rules,
        "merge_file": merge_file,
        "merge_header_file": None,
        "output_file": output_file,
        "output_header_file": None,
        "features_to_process": [],
        "feature_to_predict": None,
        "ignore_features": [],
        "df_json": {}
    }

    if not merge_file:
        log.error("missing merge_file - stopping")
        save_node["status"] = INVALID
        return save_node
    if not output_file:
        log.error("missing output_file - stopping")
        save_node["status"] = INVALID
        return save_node

    log.info("build_csv - START")

    common_headers, \
        headers_dict = find_all_headers(
                            pipeline_files=pipeline_files)

    log.info(("num common_headers={} headers={}")
             .format(len(common_headers),
                     common_headers))

    # since the headers can be different we rebuild a new one:

    hdrs = {}
    for h in common_headers:
        hdrs[h] = None

    features_to_process = []
    feature_to_predict = None
    ignore_features = []

    set_if_above = None
    labels = []
    label_values = []
    if label_rules:
        set_if_above = label_rules["set_if_above"]
        labels = label_rules["labels"]
        label_values = label_rules["label_values"]

    all_rows = []
    num_done = 0
    total_files = len(pipeline_files)
    for c in pipeline_files:
        log.info(("merging={}/{} csv={}")
                 .format(num_done,
                         total_files,
                         c))
        cf = pd.read_csv(c)
        log.info((" processing rows={}")
                 .format(len(cf.index)))
        for index, row in cf.iterrows():
            valid_row = True
            new_row = copy.deepcopy(hdrs)
            new_row["src_file"] = c
            for k in hdrs:
                if k in row:
                    new_row[k] = row[k]
            # end of for all headers to copy in

            if label_rules:
                test_rand = random.randint(0, 100)
                if test_rand > set_if_above:
                    new_row["label_value"] = label_values[1]
                    new_row["label_name"] = labels[1]
                else:
                    new_row["label_value"] = label_values[0]
                    new_row["label_name"] = labels[0]
            # end of applying label rules

            if valid_row:
                all_rows.append(new_row)
        # end of for all rows in this file

        num_done += 1
    # end of building all files into one list

    log.info(("merged rows={} generating df")
             .format(len(all_rows)))
    df = pd.DataFrame(all_rows)
    log.info(("df rows={} headers={}")
             .format(len(df.index),
                     df.columns.values))

    if output_file:
        log.info(("writing merge_file={}")
                 .format(merge_file))
        df.to_csv(merge_file,
                  sep=',',
                  encoding='utf-8',
                  index=False)
        log.info(("done writing merge_file={}")
                 .format(merge_file))

        merge_header_file = "{}/merge_{}".format(
            "/".join(merge_file.split("/")[:-1]),
            headers_filename)
        log.info(("dropping merge headers file={}")
                 .format(merge_header_file))
        header_data = {"headers": list(df.columns.values),
                       "output_type": "merge",
                       "pipeline_files": pipeline_files,
                       "post_proc_rules": post_proc_rules,
                       "label_rules": label_rules,
                       "created": rnow()}
        with open(merge_header_file, "w") as otfile:
            otfile.write(str(ppj(header_data)))

        save_node["merge_file"] = merge_file
        save_node["merge_header_file"] = merge_header_file

        save_node["df_json"] = df.to_json()

        if post_proc_rules:

            output_header_file = ""

            feature_to_predict = "label_name"
            features_to_process = []
            ignore_features = []
            if label_rules:
                ignore_features = [feature_to_predict]

            if "drop_columns" in post_proc_rules:
                for p in post_proc_rules["drop_columns"]:
                    if p in headers_dict:
                        ignore_features.append(p)
                # post proce filter more features out
                # for non-int/float types

                for d in df.columns.values:
                    add_this_one = True
                    for i in ignore_features:
                        if d == i:
                            add_this_one = False
                            break
                    if add_this_one:
                        features_to_process.append(d)
                # for all df columns we're not ignoring...
                # add them as features to process

                log.info(("writing DROPPED output_file={} "
                          "features_to_process={}"
                          "ignore_features={}"
                          "predict={}")
                         .format(output_file,
                                 features_to_process,
                                 ignore_features,
                                 feature_to_predict))

                df.drop(
                    columns=ignore_features
                ).to_csv(output_file,
                         header=False,
                         sep=',',
                         encoding='utf-8',
                         index=False)

                output_header_file = "{}/output_{}".format(
                    "/".join(output_file.split("/")[:-1]),
                    headers_filename)
                log.info(("dropping headers file={}")
                         .format(output_header_file))
                header_data = {"headers": list(df.columns.values),
                               "output_type": "output",
                               "pipeline_files": pipeline_files,
                               "post_proc_rules": post_proc_rules,
                               "label_rules": label_rules,
                               "features_to_process": features_to_process,
                               "feature_to_predict": feature_to_predict,
                               "ignore_features": ignore_features,
                               "created": rnow()}
                with open(output_header_file, "w") as otfile:
                    otfile.write(str(ppj(header_data)))
            else:

                for d in df.columns.values:
                    add_this_one = True
                    for i in ignore_features:
                        if d == i:
                            add_this_one = False
                            break
                    if add_this_one:
                        features_to_process.append(d)
                # for all df columns we're not ignoring...
                # add them as features to process

                log.info(("writing output_file={} "
                          "features_to_process={}"
                          "ignore_features={}"
                          "predict={}")
                         .format(output_file,
                                 features_to_process,
                                 ignore_features,
                                 feature_to_predict))

                df.to_csv(output_file,
                          header=False,
                          sep=',',
                          encoding='utf-8',
                          index=False)

                output_header_file = "{}/output_{}".format(
                    "/".join(output_file.split("/")[:-1]),
                    headers_filename)
                log.info(("dropping headers file={}")
                         .format(output_header_file))
                header_data = {"headers": list(df.columns.values),
                               "output_type": "output",
                               "pipeline_files": pipeline_files,
                               "post_proc_rules": post_proc_rules,
                               "label_rules": label_rules,
                               "features_to_process": features_to_process,
                               "feature_to_predict": feature_to_predict,
                               "ignore_features": ignore_features,
                               "created": rnow()}
                with open(output_header_file, "w") as otfile:
                    otfile.write(str(ppj(header_data)))

            # end of if/else

            save_node["output_file"] = output_file
            save_node["output_header_file"] = output_header_file

            log.info(("done writing output_file={}")
                     .format(output_file))
        # end of post_proc_rules

        save_node["status"] = VALID
    # end of writing the file

    save_node["features_to_process"] = features_to_process
    save_node["feature_to_predict"] = feature_to_predict
    save_node["ignore_features"] = ignore_features

    log.info("build_csv - END")

    return save_node
# end of build_csv


def find_all_pipeline_csvs(
        csv_glob_path="/opt/datasets/**/*.csv"):

    log.info("finding pipeline csvs in dir={}".format(csv_glob_path))

    pipeline_files = []

    for csv_file in glob.iglob(csv_glob_path,
                               recursive=True):
        log.info(("adding file={}")
                 .format(csv_file))
        pipeline_files.append(csv_file)
    # end of for all csvs

    log.info(("pipeline files={}")
             .format(len(pipeline_files)))

    return pipeline_files
# end of find_all_pipeline_csvs


output_dir = ev(
                "OUTPUT_DIR",
                "/tmp")
output_file = ev(
                "OUTPUT_FILE",
                "{}/only_attack_scans.csv".format(
                    output_dir))
merge_file = ev(
                "MERGE_FILE",
                "{}/merge_only_attack_scans.csv".format(
                    output_dir))
dataset_dir = ev(
                "DS_DIR",
                "/opt/datasets")
csv_glob_path = ev(
                "DS_GLOB_PATH",
                "{}/*/*.csv".format(
                    dataset_dir))

pipeline_files = find_all_pipeline_csvs(
                    csv_glob_path=csv_glob_path)

post_proc_rules = {
    "drop_columns": [
        "src_file",
        "raw_id",
        "raw_load",
        "raw_hex_load",
        "raw_hex_field_load",
        "pad_load"
    ],
    "predict_feature": "label_name"
}

label_rules = {
    "set_if_above": 85,
    "labels": ["not_attack", "attack"],
    "label_values": [0, 1]
}

log.info("building csv")

save_node = build_csv(
    pipeline_files=pipeline_files,
    merge_file=merge_file,
    output_file=output_file,
    post_proc_rules=post_proc_rules,
    label_rules=label_rules)

if save_node["status"] == VALID:
    log.info("Successfully process datasets:")

    if ev("SHOW_SUMMARY",
          "1") == "1":
        log.info(("Merge csv: {}")
                 .format(save_node["merge_file"]))
        log.info(("Merge headers: {}")
                 .format(save_node["merge_header_file"]))
        log.info(("Output csv: {}")
                 .format(save_node["output_file"]))
        log.info(("Output headers: {}")
                 .format(save_node["output_header_file"]))
        log.info("------------------------------------------")
        log.info(("Predicting Feature: {}")
                 .format(save_node["feature_to_predict"]))
        log.info(("Features to Process: {}")
                 .format(ppj(save_node["features_to_process"])))
        log.info(("Ignored Features: {}")
                 .format(ppj(save_node["ignore_features"])))
        log.info("------------------------------------------")
    # end of show summary

    log.info("")
    log.info("done saving csv:")
    log.info("{}".format(save_node["merge_file"]))
    log.info("")
else:
    log.info("Failed to process datasets")
# end if/else
